#!/usr/bin/perl
#
# add2muwed.pl - add mp3 files to your database
#
# Copyright (c) by Andreas S. Krebs (andreas.krebs@web.de)
#
# this script tries to get the information for the muwed from the mp3 file in the following way:
# 1. It tries to split the filename into "ARTIST - SONG.mp3"
# 2. It tries to read the ID3 tags from the mp3 file
# 3. If the artist and song title constructed from the filename is longer than that from the ID3 tag it's used 
# 4. It tries to read the mp3 header from the mp3 file
#
# NOTE:
# this script expects the filename given to it relative to your mp3 root dir, otherwise the database entries
# will be incorrect
#
# TODO:
# -adding support to update the database, if an entry already exists
# 

use Pg;
use MP3::Info;
use MP3::Info qw(:all);

use_winamp_genres();

$DBNAME="muwed";
$DBUSER="postgres";

$USE_ALBUMTAG=1;
$USE_NOTETAG=0;
$USE_FILENAME=1;

# try to connect to the database
$conn=Pg::connectdb("dbname=$DBNAME user=$DBUSER");
( PGRES_CONNECTION_OK eq $conn->status )
    and print "Pg::connectdb ........... ok\n"
    or  die   "Pg::connectdb ........... not ok: ", $conn->errorMessage;

foreach $file (@ARGV)
{
  processFile($file);
}

1;

sub processFile
{
  local($file)=shift(@_);
  $file=~s/^.\///;
  local($inx, $line, $title, $artist, $rem, $tags);

  $efile=$file;
  $efile=~ s/`/\\`/g;
  $line=`basename \"$efile\"`;

  print "processing: $file\n";  
  
  $inx=index($line, ".mp3");
  $artist="";
  $title="";
  if($inx > 0 && $USE_FILENAME==1)
  {
    $line=substr($line, 0, $inx);
    ($fartist, $ftitle)=split(' - ', $line, 2);
  }

  $fartist=&removeSpaces($fartist);
  $ftitle=&removeSpaces($ftitle);

  ($title, $artist, $album, $note, $year, $genre)=&getTags($file);
  if(length($ftitle) >= length($title) && length($fartist) >= length($artist))
  {
    $title=$ftitle;
    $artist=$fartist;
  }

  if($artist ne "" && $title ne "")
  {
    ($size, $length, $br, $flags)=&getInfo($efile);
    print "$file: $artist - $title ($album): $size bytes, $length secs, $br kbs, $flags\n";

    $file=&db_escape($file);
    $title=&db_escape($title);
    $artist=&db_escape($artist);
    $album=&db_escape($album);
    $note=&db_escape($note);
    $year=&db_escape($year);
    $genre=&db_escape($genre);
    $size=&db_escape($size);
    $length=&db_escape($length);
    $br=&db_escape($br);
    $flags=&db_escape($flags);

    print("INSERT INTO songs VALUES ($file, $title, $artist, $album, $note, $year, $genre, $size, $length, $br, $flags)");
#    &pg_exec("INSERT INTO songs VALUES ($file, $title, $artist, $album, $note, $year, $genre, $size, $length, $br, $flags)");
  }
}

sub pg_exec
{
  local($query)=shift(@_);
  local($result);

  print "$query\n";
  $result=$conn->exec($query);
  print $conn->errorMessage;
}

sub db_escape
{
  local($field)=shift(@_);

  $field=~ s/'/''/g;
#  $field=~ s/`/\\`/g;

  if($field eq "")
  {
    $field="NULL";
  }
  else
  {
    $field="'$field'";
  }

  return ($field);
}

sub getTags
{
  local($file)=shift(@_);
  local($tag, $title, $artist, $album, $note, $year, $genre);

  $tag=get_mp3tag($file);

  $title=$tag->{TITLE};
  $artist=$tag->{ARTIST};

  if($USE_ALBUMTAG==1)
  {
    $album=$tag->{ALBUM};
  }

  if($USE_NOTETAG==1)
  {
    $note=$tag->{COMMENT};
  }

  $year=$tag->{YEAR};
  $genre=$mp3_genres{ $tag->{GENRE} };

  return ($title, $artist, $album, $note, $year, $genre);
}

sub getInfo
{
  local($file)=shift(@_);
  local($info, $size, $length, $br, $flags);

  $info=get_mp3info($file);
  $flags=0;

  $size=$info->{SIZE};
  $length=$info->{SECS};
  $length=~ s/\..*//g;

  $flags|=$info->{LAYER};
  $br=$info->{BITRATE};

  if($info->{STEREO})
  {
    $flags|=4;
  }

  if($info->{MODE} eq 1)	# joint stereo ?
  {
    $flags|=8;
  }

  if($info->{VBR})
  {
    $flags|=16;
  }
  
  return ($size, $length, $br, $flags);
}

sub removeSpaces
{
  local($txt)=shift(@_);

  while(substr($txt, 0, 1) eq " " || substr($txt, 0, 1) eq "\t")
  {
    $txt=substr($txt, 1);
  }

  while(substr($txt, -1, 1) eq " " || substr($txt, -1, 1) eq "\t")
  {
    $txt=substr($txt, 0, -1);
  }

  $txt=~s/"/\\"/g;

  return $txt;
}

